package cs155.hw3;

/**
 This class represents a camera.
 It is initially at (0,0,0) facing down the z-axis (i.e. in the direction (0,0,-1))
 The class has several transform methods (translate, rotateX, rotateY, rotateZ)
 that allow one to move the camera to another viewpoint.
 The camera object is responsible for generating rays corresponding to screen coordinates
 which are doubles u,v in the range [-1,1]
 **/

public class Camera3D {

    private Transform3D M;

    public double screenDist = -2d;

    public Camera3D (Point3D position) {
        M = Transform3D.translateOp(position);
    }

    public Camera3D translate(Point3D p) {
	M = M.mult(Transform3D.translateOp(p.x,p.y,p.z));
	//M = Transform3D.translateOp(p.x,p.y,p.z).mult(M);
        return this;
    }

    public Camera3D rotateX(double angle){
	M = M.mult(Transform3D.rotateXOp(angle));
	//M = Transform3D.rotateXOp(angle).mult(M);
        return this;
    }

    public Camera3D rotateY(double angle){
	M = M.mult(Transform3D.rotateYOp(angle));
	//M = Transform3D.rotateYOp(angle).mult(M);
        return this;
    }

    public Camera3D rotateZ(double angle){
	M = M.mult(Transform3D.rotateZOp(angle));
	//M = Transform3D.rotateZOp(angle).mult(M);
        return this;
    }

    /**
      Convert screen coordinates into a ray.
      Given doubles u,v which are both between -1 and 1, this method returns
      the corresponding ray which starts at the location of the camera and then
      passes through the virtual screen which is 1 unit if front of the camera
      and at position (u,v) relative to the center of that plane.
     **/
    public Ray3D generateRay(double u, double v) {
      Ray3D r = new Ray3D(new Point3D(u,v,screenDist));
      return M.apply(r);
    }

}

