package cs155.hw3;

/** 
  NewRayCanvas3D.java
**/


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.image.*;

/**
 *   This class creates a panel where one can draw pixels
 *   @author Timothy J. Hickey, tim@cs.brandeis.edu http://www.cs.brandeis.edu/~tim 
 **/


public class NewRayCanvas3D extends Panel implements Canvas3D {
    public Scene3D scene;
    public BufferedImage buffer;
    public Graphics bufferg;
    private int width,height;
    int bufferwidth,bufferheight;

    public int width() {return width;}
    public int height() {return height;}
   
    Dimension size;
 
    public NewRayCanvas3D(Scene3D scene, int w, int h) {
        super();
        this.scene = scene;
        width=bufferwidth=(w>0)?w:1; 
        height=bufferheight=(h>0)?h:1;
        size = new Dimension(w,h);
}

    public Dimension preferredSize() {return size;}

    public void update(Graphics g) { paint(g); } // don't let Java blank the screen

    /* if new window needs larger buffer then expand image buffer */
    void resizeCanvas(Dimension s) {
        int w=s.width;
        int h=s.height;
        boolean resize= ((w>bufferwidth)||(h>bufferheight));
        width=w; height=h;
        bufferwidth  = (w<bufferwidth) ?bufferwidth: w;
        bufferheight = (h<bufferheight)?bufferheight:h;
        if (resize) {
            BufferedImage buffer2 = buffer;
            Graphics bufferg2 = bufferg;
            buffer = new BufferedImage(bufferwidth,bufferheight,BufferedImage.TYPE_INT_ARGB);
            bufferg = buffer.getGraphics();
            bufferg.drawImage(buffer2,0,0,this);
        }
    }

    /**
       this draws a pixel of a specified color on the offscreen buffer
     **/
    public void drawPixel(int x, int y, Color c) { 
        buffer.setRGB(x,y,c.getRGB());
    }

    private boolean drawing=false;
    public boolean active=false;

    /**
       This procedure is called when the panel needs to be redrawn ..
       It causes the scene to be drawn on the panel ...
     **/
    public void paint(Graphics g) {
      // handle window resizes....
        if (buffer==null)
             buffer = new BufferedImage(width,height,BufferedImage.TYPE_INT_ARGB);
        if (bufferg==null) bufferg = buffer.getGraphics();
        if (!(size.equals(this.size()))) resizeCanvas(this.size());
        if (buffer!=null) {
	    g.drawImage(buffer,0,0,this); // copy offscreen buffer to screen
        }  // close if

     }

    public void refresh() {
        if (!drawing && active){
           drawing=true;
           RayTracer3D.drawScene(scene,this);
           drawing=false;
	            }

      if (buffer!=null) {
	    this.getGraphics().drawImage(buffer,0,0,this); // copy offscreen buffer to screen
        }  // close if
    }

}

 

   

