package cs155.hw3;

public class Transform3D {
    public double[][] trans;

    public Transform3D() {
	trans = new double[][]{
         {1d, 0, 0, 0},
	 { 0,1d, 0, 0},
	 { 0, 0,1d, 0},
	 { 0, 0, 0,1d}};
    }

    public String toString() {
    return 
 "["+trans[0][0]+","+trans[0][1]+","+trans[0][2]+","+trans[0][3]+"\n"+
     trans[1][0]+","+trans[1][1]+","+trans[1][2]+","+trans[1][3]+"\n"+
     trans[2][0]+","+trans[2][1]+","+trans[2][2]+","+trans[2][3]+"\n"+
     trans[3][0]+","+trans[3][1]+","+trans[3][2]+","+trans[3][3]+"]\n";
    }

    public Transform3D mult(Transform3D M) {
	Transform3D r = new Transform3D();
        for(int i=0; i<4;i++) r.trans[i][i]=0;
        for(int i=0;i<4;i++)
	    for(int j=0;j<4;j++)
		for(int k=0; k<4; k++)
		    r.trans[i][j] += trans[i][k]*M.trans[k][j];
        return(r);
    }

    public static Transform3D mult(Transform3D A,Transform3D M) {
	Transform3D r = new Transform3D();
        for(int i=0; i<4;i++) r.trans[i][i]=0;
        for(int i=0;i<4;i++)
	    for(int j=0;j<4;j++)
		for(int k=0; k<4; k++)
		    r.trans[i][j] += A.trans[i][k]*M.trans[k][j];
        return(r);
    }

    public Point3D apply(Point3D p) {
	Point3D result = new Point3D(0d,0d,0d);
        result.x = 
           trans[0][0]*p.x + 
           trans[0][1]*p.y + 
           trans[0][2]*p.z + 
           trans[0][3]*p.w;
        result.y = 
           trans[1][0]*p.x + 
           trans[1][1]*p.y + 
           trans[1][2]*p.z + 
           trans[1][3]*p.w;
        result.z = 
           trans[2][0]*p.x + 
           trans[2][1]*p.y + 
           trans[2][2]*p.z + 
           trans[2][3]*p.w;
        result.w = 
           trans[3][0]*p.x + 
           trans[3][1]*p.y + 
           trans[3][2]*p.z + 
           trans[3][3]*p.w;
        return(result);
    }


    public Transform3D invertTrans() {
	double[] v = new double[16];
        for(int i=0; i<4; i++)
	    for(int j=0; j<4; j++)
		v[i+4*j]=trans[i][j];
        javax.vecmath.Matrix4d m = 
	    new javax.vecmath.Matrix4d(v);
        Transform3D n = new Transform3D();
        m.invert();
        for(int i=0; i<4; i++)
	    for(int j=0; j<4; j++)
		n.trans[i][j] = m.getElement(i,j);
	return n;
    }

    public Point3D applyTranspose(Point3D p) {
	Point3D result = new Point3D(0d,0d,0d);
        result.x = 
           trans[0][0]*p.x + 
           trans[1][0]*p.y + 
           trans[2][0]*p.z + 
           trans[3][0]*p.w;
        result.y = 
           trans[0][1]*p.x + 
           trans[1][1]*p.y + 
           trans[2][1]*p.z + 
           trans[3][1]*p.w;
        result.z = 
           trans[0][2]*p.x + 
           trans[1][2]*p.y + 
           trans[2][2]*p.z + 
           trans[3][2]*p.w;
        result.w = 
           trans[0][3]*p.x + 
           trans[1][3]*p.y + 
           trans[2][3]*p.z + 
           trans[3][3]*p.w;
        return(result);
    }

    public Transform3D transpose() {
	Transform3D t = new Transform3D();
	for(int i=0;i<4;i++)
	    for(int j=0;j<4;j++)
		t.trans[i][j]=trans[j][i];
	return t;
    }



    /* **************************************************************** */
    // instance methods M.op1().op2().op3() ==> op3(op2(op1(M))) ...

    public Transform3D translate(double x, double y, double z) {
	return this.mult(translateOp(x,y,z));
    }

    public Transform3D scale(double x, double y, double z) {
        return this.mult(scaleOp(x,y,z));
    }

    public Transform3D rotateZ(double angle){
        return this.mult(rotateZOp(angle));
    }

    public Transform3D rotateY(double angle){
        return this.mult(rotateYOp(angle));
    }

    public Transform3D rotateX(double angle){
        return this.mult(rotateXOp(angle));
    }

    /* **************************************************************** */


    public static Transform3D translateOp(Point3D p) {
	return translateOp(p.x, p.y, p.z);
    }

    public static Transform3D translateOp(double x, double y, double z) {
	Transform3D r = new Transform3D();
	r.trans[0][3] = x;
	r.trans[1][3] = y;
	r.trans[2][3] = z;
        return(r);
    }


    public static Transform3D scaleOp(double x, double y, double z) {
	Transform3D r = new Transform3D();
	r.trans[0][0] = x;
	r.trans[1][1] = y;
	r.trans[2][2] = z;
        return(r);
    }

    public static Transform3D rotateZOp(double angle){
	Transform3D r = new Transform3D();
        double c = Math.cos(angle);
        double s = Math.sin(angle);
	r.trans[0][0] = c;
	r.trans[0][1] = s;
	r.trans[1][0] = -s;
	r.trans[1][1] = c;
        return(r);
    }

    public static Transform3D rotateYOp(double angle){
	Transform3D r = new Transform3D();
        double c = Math.cos(angle);
        double s = Math.sin(angle);
	r.trans[0][0] = c;
	r.trans[0][2] = -s;
	r.trans[2][0] = s;
	r.trans[2][2] = c;
        return(r);
    }

    public static Transform3D rotateXOp(double angle){
	Transform3D r = new Transform3D();
        double c = Math.cos(angle);
        double s = Math.sin(angle);
	r.trans[1][1] = c;
	r.trans[1][2] = s;
	r.trans[2][1] = -s;
	r.trans[2][2] = c;
        return(r);
    }

    public Ray3D apply(Ray3D r) {
        r.d.w=0;
        r.p.w=1;
        return new Ray3D(this.apply(r.p), this.apply(r.d));
    }   

}
